@@ -1,11 +1,11 @@
 from MindMapBBS import MindMapBBS
-from MindMapBBS import manage_addFunctionForm, manage_addFunction
+from MindMapBBS import manage_addMindMapBBSForm, manage_addMindMapBBS
 from ImageFile import ImageFile
 
 def initialize(registrar):
     registrar.registerClass(
         MindMapBBS,
-        constructors = (manage_addFunctionForm, manage_addFunction),
+        constructors = (manage_addMindMapBBSForm, manage_addMindMapBBS),
         icon='www/icon.png',
     )
 
@@ -16,4 +16,7 @@ misc_ = {
     "back":    ImageFile('www/back.png', globals()),
     "up":      ImageFile('www/up.png', globals()),
     "down":    ImageFile('www/down.png', globals()),
+    "newmmb":  ImageFile('www/newmmb.png', globals()),
+    "link":    ImageFile('www/link.png', globals()),
+    "mmbLink": ImageFile('www/mmbLink.png', globals()),
     }
